package sv.kernel;

import java.awt.*;
import java.io.*;
import java.util.*;

import sv.util.*;

/** 
 * DataSet3D.java
 * Define isosurface data in each time
 * @version 1.0   10/15/97
 * @author Byeongseob Ki
 */

public class DataSet3D extends DataSet {

  /**
   * point data in each time
   */
  public TimeData3D[] timeData;

  public static int XSKIPED = 0;
  public static int YSKIPED = 1;
  public static int ZSKIPED = 2;

  
  private int data_version;
  private String source_hostname;
  private int source_port;
  public String sourceData;

  /**
   * constructor for 3d dataset
   */
  public DataSet3D(String title, int version, 
                        String hostname, int port, TimeData3D td) {
      setRank(3);
      this.title = title;
      data_version = version;
      source_hostname = hostname;
      source_port = port;
      timeArr = new float[1];
      timeData = new TimeData3D[1];

      timeData[0] = td;
      timeArr[0] = td.getTime();
  }

  /**
   * constructor for 3d dataset
   */
  public DataSet3D(String title, int version, 
              String hostname, int port, TimeData3D td, String fname) {
      this(title, version, hostname, port, td);
       
      if (fname.endsWith(".svb")) {
        String newfname = fname.substring(0, fname.length()-1);
        newfname = newfname + "w";
        sourceData = newfname;
      } 
  }


  /**
   * constructor for 3d dataset
   */
  public DataSet3D(String title, TimeData3D td) {
      setRank(3);
      this.title = title;
      data_version = -1;
      source_hostname = null;
      source_port = -1;
      timeArr = new float[1];
      timeData = new TimeData3D[1];

      timeData[0] = td;
      timeArr[0] = td.getTime();
  }


  /**
   * extract surface data info 
   */
  public void extractSurfaceData(TimeData3D td, int skipmode, int extraction) {
      int timesteps, dimx = 0, dimy = 0, dimz = 0, tot;
      float timeval;
      GridInfo3D grids = null;

      long pos = 0;

      if (sourceData == null) {
         System.out.println("No grid data file ...");
         return;
      }

      try {
         RandomAccessFile in = new RandomAccessFile(sourceData, "r");
 
         timesteps = in.readInt();  
         pos = 4;

         for (int i=0; i<timesteps; i++) {
            // set the file position
            pos = pos + (dimx * dimy * dimz * 4);
            in.seek(pos);
            pos = pos + 16;

            // read time value and dimension
	    timeval = in.readFloat(); 
            dimx = in.readInt(); 
            dimy = in.readInt();
            dimz = in.readInt();

            if (td.getTime() == timeval) {

               if (skipmode == XSKIPED) {
                  tot = dimy * dimz;
                  grids = new GridInfo3D(tot, XSKIPED, extraction);

                  in.skipBytes(extraction * tot * 4);
	          //System.out.println("current: " + in.getFilePointer());

                  for (int j=0; j<tot; j++) 
                     grids.setValue(j, in.readFloat());

               } else if (skipmode == YSKIPED) {
                  tot = dimx * dimz;
                  int idx = 0;
                  grids = new GridInfo3D(tot, YSKIPED, extraction);

                  for (int j=0; j<dimx; j++) {
                     in.skipBytes((dimz * extraction) * 4);
		     //System.out.println("current: " + in.getFilePointer());
 
                     for (int k=0; k<dimz; k++) {
                        grids.setValue(idx, in.readFloat());
                        idx++;
                     }
                     in.skipBytes((dimz * (dimy-extraction-1)) * 4);
                  }

               } else if (skipmode == ZSKIPED) {
                  tot = dimx * dimy;
                  int idx = 0;
                  grids = new GridInfo3D(tot, ZSKIPED, extraction);

                  for (int j=0; j<dimx; j++) {
                     for (int k=0; k<dimy; k++) {
                        in.skipBytes(extraction * 4);
	   	     //System.out.println("current: " + in.getFilePointer());
                        grids.setValue(idx, in.readFloat());
                        in.skipBytes((dimz-extraction-1) * 4);
		        //System.out.println(grids.getValue(idx));
                        idx++;
                     }
                  }
               }
               td.setSurfaceGrid(grids);
               break;
	    }
          }

          in.close();

       } catch (Exception e) {
         e.printStackTrace();
       }
  }


  // return data_version
  public int getVersion() {
    return data_version;
  }

  
  // return source_hostname
  public String getSourceHostname() {
    return source_hostname;
  }
 
  // return source_port
  public int getSourcePort() {
    return source_port;
  }


  /**
   * constructor for 3D  data (****FOR FUTURE USE***)
   * @param title title of this class
   * @param timeDataVec time data vector
   */
  public DataSet3D(String title, Vector vec) {
     setRank(3);
     this.title = title;
     noOfTime = vec.size();

     timeData = new TimeData3D[noOfTime];
     timeArr = new float[noOfTime];

     vec.copyInto(timeData);
     for (int i=0; i<noOfTime; i++) 
        timeArr[i] = timeData[i].getTime();
  }


  /** 
   * add TimeData3D to this class
   * @param timedata TimeData3D to be added
   */
  public void addTimeData(TimeData data) {
     int pos, oldsize, newsize;
     TimeData3D timedata = (TimeData3D)data;
     float newtime = timedata.getTime();

     noOfTime++;     

     TimeData3D[] tmpTimeData = timeData;
     float[] tmpTimeArr = timeArr;

     oldsize = tmpTimeArr.length;
     newsize = oldsize + 1;

     pos = -1;

     for (int i=oldsize-1; i>=0; i--) {
        if (tmpTimeData[i].getTime() < newtime) {
           pos = i;
           break;
        }
     }


     timeData = new TimeData3D[newsize];
     timeArr = new float[newsize];

     if (pos == -1) {
        timeData[0] = timedata;
        timeArr[0] = timedata.getTime();
        for (int i=0; i<oldsize; i++) {
          timeData[i+1] = tmpTimeData[i];
          timeArr[i+1] = tmpTimeArr[i];
        }
     } else if (pos == oldsize) {
        for (int i=0; i<oldsize; i++) {
          timeData[i] = tmpTimeData[i];
          timeArr[i] = tmpTimeArr[i];
        }
        timeData[oldsize] = timedata;
        timeArr[oldsize] = timedata.getTime();
     } else {
        for (int i=0; i<=pos; i++) {
           timeData[i] = tmpTimeData[i];
           timeArr[i] = tmpTimeArr[i];
        }
        timeData[pos+1] = timedata;
        timeArr[pos+1] = timedata.getTime();
        for (int i=pos+2; i<newsize; i++) {
           timeData[i] = tmpTimeData[i-1];
           timeArr[i] = tmpTimeArr[i-1];
        }
     }
  }


  /**
   * update TimeData 
   * @param data TimeData3D to be update
   * @param pos position to be updated
   */
  public void updateTimeData(TimeData3D data, int pos) {
     timeData[pos] = data;
  }


} // end of class






