package sv.slide;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.io.*;
import java.net.*;

public class SlideCanvas extends Canvas implements Runnable {

   private MediaTracker mediaTracker;   // to track loading of backgound image

   private Thread thread = null;	// animation thread
   private int threadDelay = 100;	// animation thread delay 
   private long threadStartTime;	// time thread started

   private Font font = null;		// default font

   Vector animatedObjects = null;	// animated objects
   int noOfObjects = 0;		        // number of objects
   int currentObjectIndex = 0;	        // current object index
   SlideAnimate currentObj;	        // current object instance
   
   boolean initiated = false;	        // true when initiated
   boolean seenall = false;
 
   Image offImage;		        // "off-screen" image
   Dimension offSize;		        // "off-screen" size
 
 
   /**
    * Init applet
    */
   public SlideCanvas() {
      initializeSlide();
   }
   
   /**
    * Create animated text vector with SlideAnimate instances.
    */
   public void createAnimates(Vector lines) {
      
      noOfObjects = 0;
      animatedObjects = new Vector();

      for (int i=0; i<lines.size(); i++) {
         animatedObjects.addElement(new SlideAnimate(this,
                 (String)lines.elementAt(i), font, 
                 getForeground(), getBackground(), 3, 2));
         noOfObjects++;
      }
      currentObjectIndex = 0;	
      currentObj = 
             (SlideAnimate)animatedObjects.elementAt(currentObjectIndex);
   }
   
   // Animate the texts
   void animate(Graphics g) {
      if (currentObj.update(g)) {
         currentObjectIndex++; 
         if (currentObjectIndex >= noOfObjects) 
            currentObjectIndex = 0;

         currentObj = 
            (SlideAnimate)animatedObjects.elementAt(currentObjectIndex);
         currentObj.reset();
      }

   }


   /**
    * Paint last animation
    */
   public void paint(Graphics g) {
      if (offImage != null) 
         g.drawImage(offImage, 0, 0, null);
   }

   public void update(Graphics g) {
      paint(g);
   }
   
   /**
    * Update one frame of animation
    */
   void updateAnimation() {
      if (!initiated) 
         return;

      // get size of applet 
      Dimension d = getSize();
      if (d.width == 0 || d.height == 0) 
         d = new Dimension(400, 240);
		 
      // Create the offscreen graphics context if required
      if ((offImage == null) || (d.width != offSize.width) ||
                                    	    (d.height != offSize.height)) {
	 // create off-screen graphics context
	 offSize = d;
         offImage = this.createImage(d.width, d.height);

	 // reset Animated Text item
	 currentObj.reset();
      }

      Graphics offGraphics = offImage.getGraphics();
	 
      // paint frame
      offGraphics.setColor(getBackground());
      offGraphics.fillRect(0, 0, d.width, d.height);
   
      // reset text background
      offGraphics.setColor(getBackground());
      offGraphics.fillRect(0, 0, offSize.width, offSize.height);
   
      // animate text
      animate(offGraphics);
      offGraphics.dispose();
    }


   /**
    * Run the loop. This method is called by class Thread.
    */
   public void run() {
      
      if (Thread.currentThread() == thread) {
	 thread.setPriority(Thread.MIN_PRIORITY);
	 if (!initiated) {
	    try {
	       Thread.sleep(50);
	    } catch (InterruptedException e) {}
	 }
      }
      
      while (Thread.currentThread() == thread) {
	 // Update one frame of animation
         updateAnimation();
         repaint();
	 
	 // Delay depending on how far we are behind (to assure
         // we really delay as much as requested).
	 try {
	    threadStartTime += threadDelay;
	    int delay = (int) Math.max(
	       threadDelay/2, threadStartTime - System.currentTimeMillis());
	    Thread.sleep(delay);
	 } catch (InterruptedException e) {
	    break;
	 }
      }
   }
   
   /**
    * Start the applet by forking an animation thread.
    */
   public void start() {
      if (thread == null) {
         thread = new Thread(this);
	 thread.start();
      }

      // remember the thread start time
      threadStartTime = System.currentTimeMillis();
   }
   
   /**
    * Stop the applet. The thread will exit because run() exits.
    */
   public void stop() {
      thread = null;
   }

   /**
    * Get offscreen image
    */
   public Image getOffImage() {
      return offImage;
   }


   /**
    * Get font from string repr. of name, style and size
    */
   public Font getSlideFont(String fontName, String fontStyle, String fontSize) {
      int style = Font.BOLD;
      int size = 24;
      
      if (fontName == null) {
	 if (this.font != null)
	    fontName = this.font.getName();
	 else
	    fontName = "TimesRoman";
      }
      
      if (fontStyle == null) {
	 if (this.font != null)
	    style = this.font.getStyle();
      }  else {
	 if (fontStyle.equalsIgnoreCase("plain"))
	    style = Font.PLAIN;
	 else if (fontStyle.equalsIgnoreCase("bold"))
	    style = Font.BOLD;
	 else if (fontStyle.equalsIgnoreCase("italic"))
	    style = Font.ITALIC;
	 else if (fontStyle.equalsIgnoreCase("bolditalic"))
	    style = Font.ITALIC|Font.BOLD;
      }
      
      if (fontSize == null) {
	 if (this.font != null)
	    size = this.font.getSize();
      } else {
	 size = Integer.parseInt(fontSize);
      }

      return new Font(fontName, style, size);
   }
   

   /**
    * Convert a Hexadecimal String with RGB-Values to Color
    * Uses aDefault, if no or not enough RGB-Values
    */
   public Color readColor(String aColor, Color aDefault) {
      if (aColor == null)
	 return aDefault;

      Integer rgbValue = null;
      try {
	 if (aColor.startsWith("#")) 
	    rgbValue = Integer.valueOf(aColor.substring(1), 16);
	 else if (aColor.startsWith("0x")) 
	    rgbValue = Integer.valueOf(aColor.substring(2), 16);
	 else
	    // assume symbolic color name
	    rgbValue = Integer.valueOf(SlideColorSupport.lookup(aColor), 16);
      } catch (NumberFormatException e) {
	 rgbValue = null;
      }
      
      if (rgbValue == null)
	 return aDefault;
      
      return new Color(rgbValue.intValue());
   }

   //  parse config and data files
   private void initializeSlide() {
      Vector lineVector = new Vector();

      font = new Font("TimesRoman", Font.BOLD, 24);
      setBackground(new Color(0, 200, 200));
      setForeground(Color.black);

      String[] texts = {
"<20><down><shadow><bgImage=java-powered.gif>Thank you for using Scivis!<fade>",
"<20><typed>Created by\nScott Klasky\nand\nByeongseob Ki<up>",
"<left><engrave>Northeast Parallel Architecture Center<left>",
"<20><fade><color=#FF0000>Features:<fade>",
"<20><typed>Client/Server model\nUser defineable filter\nCollaboration\nInteractive interface<down>",
"<20><up><color=#FFFF00>To go Scivis home page\nclick here<down>"
      };

      for (int i=0; i<texts.length; i++)
         lineVector.addElement(texts[i]);

      // create animated texts
      createAnimates(lineVector);
      initiated = true;
   }


} // end of class

